from Globals import DTMLFile, InitializeClass
import string
from OFS.CopySupport import cookie_path, _cb_decode
import user
import AccessControl
import utilities
from cgi import escape

class proxy:
    """Proxies catalog brains."""

    def __init__(self, object):
        self.object = object

    def __getattr__(self, name):
        if hasattr(self.object.aq_base, name):
            return getattr(self.object, name)
        else:
            return getattr(self.object.getObject(), name)

class lazy:
    """Wrapper around catalog search results."""

    def __init__(self, results):
        self.results_ = results

    def __getattr__(self, name):
        return getattr(self.results_, name)

    def __getitem__(self, index):
        return proxy(self.results_[index])

    def __getslice__(self, start, end):
        if end > len(self):
            end = len(self)
        return map(self.__getitem__, range(start, end))

class base:
    """Mixin class used to provide common functionality."""

    security = AccessControl.ClassSecurityInfo()

    security.declareProtected('Use Issue Manager', 'search', 'remote',
                              'search_remote', 'issue_tree', 'issue_listing',
                              'standard_html_footer', 'settings')
    search = DTMLFile('issue_search', globals())
    remote = DTMLFile('remote', globals())
    search_remote = DTMLFile('search_remote', globals())
    issue_tree = DTMLFile('issue_tree', globals())
    issue_listing = DTMLFile('issue_listing', globals())
    standard_html_footer = DTMLFile('standard_html_footer', globals())
    settings = DTMLFile('settings', globals())

    security.declarePrivate('on_access')
    def on_access(self):
        """Method called each time an object is accessed."""
        self.handle_session()
        if self.meta_type == 'Issue':
            self.REQUEST.SESSION['last_visited_issue'] = self.id
        else:
            if not self.REQUEST.SESSION.has_key('last_visited_issue'):
                self.REQUEST.SESSION['last_visited_issue'] = None

    security.declarePrivate('create_hyperlink')
    def create_hyperlink(self, *arguments, **keywords):
        """Creates a hyperlink."""
        return utilities.create_hyperlink(*arguments, **keywords)

    security.declarePrivate('get_parents')
    def get_parents(self, stop=None):
        """Returns a list of parent objects."""
        reference = self
        parents = []
        while 1:
            try:
                parent = reference.getParentNode()
            except AttributeError:
                # We're at the end of the line
                break
            parents.append(parent)
            if stop:
                if stop(parent):
                    break
            reference = parent
        return parents

    security.declarePrivate('render_breadcrumbs')
    def render_breadcrumbs(self, cut=0):
        """Renders breadcrumbs."""
        parents = self.REQUEST['PARENTS']
        breadcrumbs = []
        for parent in parents:
            # This conditional is needed to play nice with
            # objects that don't inherit from this class.
            if hasattr(parent.aq_base, 'get_title'):
                title = parent.get_title()
            else:
                if parent.title: title = parent.title
                else: title = parent.id.capitalize()
            breadcrumbs.append(self.create_hyperlink(
                parent.absolute_url(), title))
            if parent.meta_type == 'Issue Manager':
                break
        breadcrumbs.reverse()
        if cut:
            breadcrumbs = breadcrumbs[:-cut]
        return """<div id="breadcrumbs">""" + \
               string.join(breadcrumbs, ' :: ') + \
               """</div>"""

    security.declarePrivate('render_context_breadcrumbs')    
    def render_context_breadcrumbs(self):
        """Renders breadcrumbs based on context, for the search page."""
        parents = self.get_parents()
        breadcrumbs = []
        for parent in parents:
            if parent.meta_type == 'Issue Manager':
                break
            breadcrumbs.append(self.create_hyperlink(
                parent.absolute_url(), parent.get_title()))
        breadcrumbs.reverse()
        return string.join(breadcrumbs, ' :: ')

    security.declarePrivate('render_javascript_breadcrumbs')    
    def render_javascript_breadcrumbs(self):
        """Renders breadcrumbs used in javascript."""
        parents = self.get_parents()
        breadcrumbs = []
        for parent in parents:
            breadcrumbs.append(escape(parent.get_title()))
            if parent.meta_type == 'Issue Manager':
                break
        breadcrumbs.reverse()
        return """window.status='""" + \
               string.join(breadcrumbs, ' :: ') + \
               "'; return true"

    security.declarePrivate('get_title')
    def get_title(self):
        """Returns a descriptive title."""
        if self.title:
            title = self.title
        else:
            title = self.meta_type
        return title

    security.declarePrivate('get_search_results')
    def get_search_results(self):
        """Returns search results, based on REQUEST."""
        if not self.REQUEST.SESSION.has_key('show_deleted'):
            self.REQUEST.SESSION['show_deleted'] = 0
        if self.REQUEST.SESSION['show_deleted']:
            deleted = [0,1]
        else:
            deleted = 0
        search = self.REQUEST['search_string']
        if self.REQUEST.has_key('types'):
            types = self.REQUEST['types']
        else:
            types = self.get_issue_types()
        if self.REQUEST.has_key('states'):
            states = self.REQUEST['states']
        else:
            states = self.get_issue_states()
        if self.REQUEST.has_key('path'):
            path = self.REQUEST['path']
        else:
            path = ''
        if self.REQUEST.has_key('importance'):
            importance = map(lambda x: int(x),
                             self.REQUEST['importance'])
        else:
            importance = map(lambda x: x[0], self.get_importance_levels())
        if self.REQUEST.has_key('relative_state'):
            return self.catalog_search(get_all_text=search, type=types,
                                       get_relative_state=states, path=path, deleted=deleted,
                                       importance=importance)
        else:
            return self.catalog_search(get_all_text=search, type=types,
                                       state=states, path=path, deleted=deleted,
                                       importance=importance)

    security.declarePrivate('get_all_text')
    def get_all_text(self):
        """Returns all text on the object."""
        return self.title + " " + self.contents

    security.declarePrivate('get_issues')
    def get_issues(self):
        """Returns a list of the (non deleted) Issues contained."""
        states = self.get_issue_states()
        if not self.REQUEST.SESSION.has_key('show_deleted'):
            self.REQUEST.SESSION['show_deleted'] = 0
        if self.REQUEST.SESSION['show_deleted']:
            deleted = [0,1]
        else:
            deleted = 0
        return self.catalog_search(
            get_parent_url=self.absolute_url(relative=1),
            state=states, sort_on=self.get_sort_index(),
            deleted=deleted)

    security.declarePrivate('get_open_suspended_issues')
    def get_open_suspended_issues(self):
        """Returns a list of the (non deleted), open or suspended Issues contained."""
        states = ['open', 'suspended']
        if not self.REQUEST.SESSION.has_key('show_deleted'):
            self.REQUEST.SESSION['show_deleted'] = 0
        if self.REQUEST.SESSION['show_deleted']:
            deleted = [0,1]
        else:
            deleted = 0
        return self.catalog_search(
            get_parent_url=self.absolute_url(relative=1),
            state=states, sort_on=self.get_sort_index(),
            deleted=deleted)

    security.declarePrivate('get_closed_discarded_issues')
    def get_closed_discarded_issues(self):
        """Returns a list of the closed and discarded Issues contained."""
        states = ['closed', 'discarded']
        if not self.REQUEST.SESSION.has_key('show_deleted'):
            self.REQUEST.SESSION['show_deleted'] = 0
        if self.REQUEST.SESSION['show_deleted']:
            deleted = [0,1]
        else:
            deleted = 0
        return self.catalog_search(
            get_parent_url=self.absolute_url(relative=1),
            state=states, sort_on=self.get_sort_index(),
            deleted=deleted)

    security.declarePrivate('get_deleted_issues')
    def get_deleted_issues(self):
        """Returns a sequence of deleted issues."""
        return self.catalog_search(
            get_parent_url=self.absolute_url(relative=1),
            deleted=1)

    security.declarePrivate('get_number_of_issues')
    def get_number_of_issues(self):
        """Returns the number of Issues contained."""
        return len(self.get_issues())

    security.declareProtected('Use Issue Manager', 'manage_cut')
    def manage_cut(self, ids=None):
        """Cuts selected objects."""
        copy_data = self.manage_cutObjects(ids=ids)
        self.REQUEST.RESPONSE.setCookie('__cp', copy_data,
                                        path='%s' % cookie_path(self.REQUEST))
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Manager', 'manage_paste')
    def manage_paste(self):
        """Pastes cut objects."""
        # Move the objects and reindex old and new
        # parent and the moved objects.
        cb = _cb_decode(self.REQUEST['__cp'])
        objects = cb[1]
        ids = map(lambda x: x[-1], objects)
        objects = map(lambda x: string.join(x, '/'), objects)
        objects = map(lambda x, self=self: self.unrestrictedTraverse(x), objects)
        parent = objects[0].getParentNode()
        self.manage_pasteObjects(cb_copy_data=self.REQUEST['__cp'])
        if parent.meta_type != 'Issue Manager':
            parent.index_object()
        for id in ids:
            self[id].index_object()
        if self.meta_type != 'Issue Manager':
            self.index_object()
        self.REQUEST['RESPONSE'].setCookie('__cp', 'deleted',
                                      path='%s' % cookie_path(self.REQUEST),
                                      expires='Wed, 31-Dec-97 23:59:59 GMT')
        self.REQUEST['__cp'] = None
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Manager', 'manage_delete')
    def manage_delete(self, ids=[]):
        """Deletes the specified objects."""
        for id in ids:
            self[id].deleted = 1
            self[id].index_object()
            for issue in self[id].get_issues():
                issue.manage_delete(ids=map(
                  lambda issue: issue.id, issue.get_issues()))
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Manager', 'manage_undelete')
    def manage_undelete(self, ids=[]):
        """Undeletes the specified objects."""
        for id in ids:
            self[id].deleted = 0
            self[id].index_object()
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Manager', 'mark')
    def mark(self, ids=[]):
        """Marks the given ids or this object."""
        session = self.get_session()
        if ids:
            session.add_marks(ids)
        else:
            session.add_marks(self.id)
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Manager', 'unmark')
    def unmark(self, ids=[]):
        """Unmarks the given ids or this object."""
        session = self.get_session()
        if ids:
            session.remove(ids=ids)
        else:
            session.remove(ids=[self.id])
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declarePrivate('get_parent_url')
    def get_parent_url(self):
        """Returns the parents URL."""
        return self.getParentNode().absolute_url(relative=1)

    security.declarePrivate('catalog_search')
    def catalog_search(self, *arguments, **keywords):
        """Does a catalog search, returning proxied objects."""
        return lazy(self.Catalog(*arguments, **keywords))

    security.declarePrivate('get_user')
    def get_user(self):
        """Returns a user instance."""
        return user.user(self.REQUEST['AUTHENTICATED_USER'])

    security.declarePrivate('get_usernames')
    def get_usernames(self):
        """Returns a list of all known usernames."""
        # Implement support for user folders higher up
        # in the hierarchy
        return self.acl_users.getUserNames()

    security.declarePrivate('render_search_importance_level_widget')
    def render_search_importance_level_widget(self):
        """Returns a widget for handling importance levels."""
        widget = ""
        if not self.REQUEST.has_key('importance'):
            self.REQUEST['importance'] = \
              map(lambda x: x[0], self.get_importance_levels()[1:])
        else:
            self.REQUEST['importance'] = \
              map(lambda x: int(x), self.REQUEST['importance'])
        for key, value in self.get_importance_levels():
            widget += "&nbsp;&nbsp;"
            if key in self.REQUEST['importance']:
                widget += """<input type="checkbox" name="importance:list"
                             value="%s" checked="checked" /> %s""" % \
                             (key, value.capitalize())
            else:
                widget += """<input type="checkbox" name="importance:list"
                             value="%s" /> %s""" % (key, value.capitalize())
            widget += "<br />"
        return widget

    security.declarePrivate('update')
    def update(self):
        """Updates self."""
        try:
            if object.state == 'deleted':
                object.deleted = 1
                object.state = 'discarded'
                object.index_object()
            if object.type == 'container':
                object.type = 'undefined'
                object.index_object()
        except AttributeError:
            pass
        if self.type == 'critical problem':
            self.type = 'problem'
            self.importance = 2

    security.declarePrivate('get_sort_index')
    def get_sort_index(self):
        """Returns the default sort index."""
        return 'get_state_importance_title_sort'

    security.declarePrivate('is_issue')
    def is_issue(self):
        """Returns true if self is an instance."""
        if self.meta_type == 'Issue':
            return 1
        else:
            return 0

InitializeClass(base)
